@@ -144,13 +144,6 @@ bool AirPressureSensor::Load(sdf::ElementPtr _sdf)
   return this->Load(sdfSensor);
 }
 
-//////////////////////////////////////////////////
-bool AirPressureSensor::Update(
-  const ignition::common::Time &_now)
-{
-  return this->Update(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool AirPressureSensor::Update(
   const std::chrono::steady_clock::duration &_now)
@@ -227,4 +220,8 @@ double AirPressureSensor::ReferenceAltitude() const
   return this->dataPtr->referenceAltitude;
 }
 
-IGN_SENSORS_REGISTER_SENSOR(AirPressureSensor)
+//////////////////////////////////////////////////
+bool AirPressureSensor::HasConnections() const
+{
+  return this->dataPtr->pub && this->dataPtr->pub.HasConnections();
+}
