@@ -132,13 +132,6 @@ bool AltimeterSensor::Load(sdf::ElementPtr _sdf)
   return this->Load(sdfSensor);
 }
 
-//////////////////////////////////////////////////
-bool AltimeterSensor::Update(
-  const ignition::common::Time &_now)
-{
-  return this->Update(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool AltimeterSensor::Update(const std::chrono::steady_clock::duration &_now)
 {
@@ -222,4 +215,8 @@ double AltimeterSensor::VerticalVelocity() const
   return this->dataPtr->verticalVelocity;
 }
 
-IGN_SENSORS_REGISTER_SENSOR(AltimeterSensor)
+//////////////////////////////////////////////////
+bool AltimeterSensor::HasConnections() const
+{
+  return this->dataPtr->pub && this->dataPtr->pub.HasConnections();
+}
