@@ -510,12 +510,6 @@ void DepthCameraSensor::SetScene(ignition::rendering::ScenePtr _scene)
   }
 }
 
-//////////////////////////////////////////////////
-bool DepthCameraSensor::Update(const ignition::common::Time &_now)
-{
-  return this->Update(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool DepthCameraSensor::Update(
   const std::chrono::steady_clock::duration &_now)
@@ -558,21 +552,23 @@ bool DepthCameraSensor::Update(
       rendering::PixelUtil::MemorySize(rendering::PF_FLOAT32_R,
       width, height));
 
-  // publish
   this->AddSequence(msg.mutable_header(), "default");
   this->dataPtr->pub.Publish(msg);
 
   // publish the camera info message
   this->PublishInfo(_now);
 
-  // Trigger callbacks.
-  try
+  if (this->dataPtr->imageEvent.ConnectionCount() > 0u)
   {
-    this->dataPtr->imageEvent(msg);
-  }
-  catch(...)
-  {
-    ignerr << "Exception thrown in an image callback.\n";
+    // Trigger callbacks.
+    try
+    {
+      this->dataPtr->imageEvent(msg);
+    }
+    catch(...)
+    {
+      ignerr << "Exception thrown in an image callback.\n";
+    }
   }
 
   if (this->dataPtr->pointPub.HasConnections() &&
@@ -638,4 +634,10 @@ double DepthCameraSensor::NearClip() const
   return this->dataPtr->near;
 }
 
-IGN_SENSORS_REGISTER_SENSOR(DepthCameraSensor)
+//////////////////////////////////////////////////
+bool DepthCameraSensor::HasConnections() const
+{
+  return (this->dataPtr->pub && this->dataPtr->pub.HasConnections()) ||
+      (this->dataPtr->pointPub && this->dataPtr->pointPub.HasConnections()) ||
+      this->dataPtr->imageEvent.ConnectionCount() > 0u;
+}
