@@ -187,13 +187,6 @@ ignition::common::ConnectionPtr Lidar::ConnectNewLidarFrame(
   return nullptr;
 }
 
-//////////////////////////////////////////////////
-bool Lidar::Update(
-  const ignition::common::Time &_now)
-{
-  return this->Update(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool Lidar::Update(const std::chrono::steady_clock::duration &/*_now*/)
 {
@@ -224,12 +217,6 @@ void Lidar::ApplyNoise()
   }
 }
 
-//////////////////////////////////////////////////
-bool Lidar::PublishLidarScan(const ignition::common::Time &_now)
-{
-  return this->PublishLidarScan(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool Lidar::PublishLidarScan(const std::chrono::steady_clock::duration &_now)
 {
@@ -459,6 +446,11 @@ int Lidar::Fiducial(const unsigned int /*_index*/) const
   return -1;
 }
 
+//////////////////////////////////////////////////
+uint32_t Lidar::VisibilityMask() const
+{
+  return this->dataPtr->sdfLidar.VisibilityMask();
+}
 
 //////////////////////////////////////////////////
 bool Lidar::IsActive() const
@@ -468,4 +460,8 @@ bool Lidar::IsActive() const
   return true;
 }
 
-IGN_SENSORS_REGISTER_SENSOR(Lidar)
+//////////////////////////////////////////////////
+bool Lidar::HasConnections() const
+{
+  return this->dataPtr->pub && this->dataPtr->pub.HasConnections();
+}
