@@ -19,11 +19,9 @@
 
 #include <ignition/common/Console.hh>
 #include <ignition/common/Profiler.hh>
-
-#include <ignition/transport/Node.hh>
-
 #include <ignition/math/Frustum.hh>
 #include <ignition/math/Helpers.hh>
+#include <ignition/transport/Node.hh>
 
 #include "ignition/sensors/SensorFactory.hh"
 #include "ignition/sensors/LogicalCameraSensor.hh"
@@ -131,13 +129,6 @@ void LogicalCameraSensor::SetModelPoses(
   this->dataPtr->models = std::move(_models);
 }
 
-//////////////////////////////////////////////////
-bool LogicalCameraSensor::Update(
-  const ignition::common::Time &_now)
-{
-  return this->Update(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool LogicalCameraSensor::Update(
   const std::chrono::steady_clock::duration &_now)
@@ -212,4 +203,9 @@ msgs::LogicalCameraImage LogicalCameraSensor::Image() const
   return this->dataPtr->msg;
 }
 
-IGN_SENSORS_REGISTER_SENSOR(LogicalCameraSensor)
+//////////////////////////////////////////////////
+bool LogicalCameraSensor::HasConnections() const
+{
+  return this->dataPtr->pub && this->dataPtr->pub.HasConnections();
+}
+
