@@ -15,17 +15,26 @@
  *
 */
 
-#include <gz/common/Profiler.hh>
+#include <ignition/common/Profiler.hh>
 
-#include <gz/rendering/Camera.hh>
+// TODO(louise) Remove these pragmas once ign-rendering is disabling the
+// warnings
+#ifdef _WIN32
+#pragma warning(push)
+#pragma warning(disable: 4251)
+#endif
+#include <ignition/rendering/Camera.hh>
+#ifdef _WIN32
+#pragma warning(pop)
+#endif
 
-#include "gz/sensors/RenderingSensor.hh"
+#include "ignition/sensors/RenderingSensor.hh"
 
 /// \brief Private data class for RenderingSensor
-class gz::sensors::RenderingSensorPrivate
+class ignition::sensors::RenderingSensorPrivate
 {
   /// \brief Pointer to the scene
-  public: gz::rendering::ScenePtr scene;
+  public: ignition::rendering::ScenePtr scene;
 
   /// \brief Manually update the rendering scene graph
   public: bool manualSceneUpdate = false;
@@ -35,7 +44,7 @@ class gz::sensors::RenderingSensorPrivate
   public: std::vector<rendering::SensorPtr::weak_type> sensors;
 };
 
-using namespace gz;
+using namespace ignition;
 using namespace sensors;
 
 //////////////////////////////////////////////////
@@ -47,7 +56,7 @@ RenderingSensor::RenderingSensor() :
 //////////////////////////////////////////////////
 RenderingSensor::~RenderingSensor()
 {
-  if (!this->dataPtr->scene || !this->dataPtr->scene->IsInitialized())
+  if (!this->dataPtr->scene)
     return;
   for (auto &s : this->dataPtr->sensors)
   {
@@ -111,4 +120,12 @@ void RenderingSensor::Render()
       rc->PostRender();
     }
   }
+
+  if (!this->dataPtr->manualSceneUpdate &&
+      !this->dataPtr->scene->LegacyAutoGpuFlush())
+  {
+    // When LegacyAutoGpuFlush = true, that function gets
+    // called per sensor, so we don't have to do anything here
+    this->dataPtr->scene->PostRender();
+  }
 }
