@@ -174,6 +174,14 @@ bool RgbdCameraSensor::Init()
   return this->Sensor::Init();
 }
 
+//////////////////////////////////////////////////
+bool RgbdCameraSensor::Load(sdf::ElementPtr _sdf)
+{
+  sdf::Sensor sdfSensor;
+  sdfSensor.Load(_sdf);
+  return this->Load(sdfSensor);
+}
+
 //////////////////////////////////////////////////
 bool RgbdCameraSensor::Load(const sdf::Sensor &_sdf)
 {
@@ -434,12 +442,6 @@ void RgbdCameraSensorPrivate::OnNewRgbPointCloud(const float *_scan,
   memcpy(this->pointCloudBuffer, _scan, pointCloudBufferSize);
 }
 
-//////////////////////////////////////////////////
-bool RgbdCameraSensor::Update(const ignition::common::Time &_now)
-{
-  return this->Update(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool RgbdCameraSensor::Update(const std::chrono::steady_clock::duration &_now)
 {
@@ -618,4 +620,11 @@ unsigned int RgbdCameraSensor::ImageHeight() const
   return this->dataPtr->depthCamera->ImageHeight();
 }
 
-IGN_SENSORS_REGISTER_SENSOR(RgbdCameraSensor)
+//////////////////////////////////////////////////
+bool RgbdCameraSensor::HasConnections() const
+{
+  return (this->dataPtr->imagePub &&
+      this->dataPtr->imagePub.HasConnections()) ||
+      (this->dataPtr->depthPub && this->dataPtr->depthPub.HasConnections()) ||
+      (this->dataPtr->pointPub && this->dataPtr->pointPub.HasConnections());
+}
