@@ -405,12 +405,6 @@ void ThermalCameraSensor::SetScene(ignition::rendering::ScenePtr _scene)
   }
 }
 
-//////////////////////////////////////////////////
-bool ThermalCameraSensor::Update(const ignition::common::Time &_now)
-{
-  return this->Update(math::secNsecToDuration(_now.sec, _now.nsec));
-}
-
 //////////////////////////////////////////////////
 bool ThermalCameraSensor::Update(
   const std::chrono::steady_clock::duration &_now)
@@ -657,4 +651,10 @@ bool ThermalCameraSensorPrivate::SaveImage(const uint16_t *_data,
   return true;
 }
 
-IGN_SENSORS_REGISTER_SENSOR(ThermalCameraSensor)
+//////////////////////////////////////////////////
+bool ThermalCameraSensor::HasConnections() const
+{
+  return (this->dataPtr->thermalPub &&
+      this->dataPtr->thermalPub.HasConnections()) ||
+      this->dataPtr->imageEvent.ConnectionCount() > 0u;
+}
